//  AuthenticatorPublicKey.java
//
//  Last modified 2/17/99 by Alan Frindell
//
//  Struct like class that implements the public key interface.  Also
// supports reading and writing to disk.

package Authenticator;

import java.security.PublicKey;
import java.math.BigInteger;
import java.io.*;

public class AuthenticatorPublicKey implements PublicKey {

  public BigInteger N;
  public BigInteger T;
  public BigInteger s;

  public AuthenticatorPublicKey(BigInteger N, BigInteger T, BigInteger s)
    {
      this.N = N;
      this.T = T;
      this.s = s;
    }

  //  Writes the public key to the given filename

  public void writeToFile(String filename) {
    try {
      FileOutputStream outfile = new FileOutputStream(filename);
      ObjectOutputStream outstream = new ObjectOutputStream(outfile);

      outstream.writeObject(this);
      outstream.close();

      outfile.close();
    } catch (Exception e) {
      System.out.println("PublicKey: " + e.getMessage());
      e.printStackTrace();
    }
  }

  
  //  Creates a public key from the one stored in the given filename
  
  public static AuthenticatorPublicKey fromFile(String filename) {
    try {
      FileInputStream infile = new FileInputStream(filename);
      ObjectInputStream instream = new ObjectInputStream(infile);

      AuthenticatorPublicKey pubKey =
	(AuthenticatorPublicKey)instream.readObject();
      instream.close();

      infile.close();

      return pubKey;
    } catch (Exception e) {
      System.out.println("PublicKey: " + e.getMessage());
      e.printStackTrace();
    }
    return null;
  }    


  //  We must implement the following methods, although they are not
  // supported and do not need to be called in our system.

  public String getAlgorithm() { 
    return "Authenticator";
  }

  public String getFormat() {
    return null;
  }

  public byte[] getEncoded() {
    return null;
  }

};
