//  ChatClientApplet.java
//
//  Last modified 2/17/99 by Alan Frindell
//
//  Chat Client capaple as running as either an applet or an application
// (although for the moment, we can't get it to run as an applet, so
// you'll have to run it as an application).
//
//  You will have to change this class significantly to add authentication
// and encryption to the chat system.

package Authenticator.Chat;

import Authenticator.*;

import java.rmi.*;

import java.awt.*;
import java.awt.event.*;
import java.applet.*;

import java.io.*;
import java.math.BigInteger;

import java.security.*;
import java.security.spec.*;
import java.security.interfaces.*;
import javax.crypto.*;
import javax.crypto.spec.*;
import javax.crypto.interfaces.*;

public class ChatClientApplet extends Applet {
  boolean isStandalone = false;
  
  String mHandle;
  Integer mClientID;
  
  ChatServer mServer;
  ChatConsumer mConsumer;
  ChatLoginPanel mLoginPanel;
  ChatSubscribePanel mSubscribePanel;
  ChatRoomPanel mChatPanel;

  CardLayout mLayout = new CardLayout();
  
  public static final int SUCCESS = 0;
  public static final int CONNECTION_REFUSED = 1;
  public static final int AUTHENTICATION_FAILED = 2;
  public static final int BAD_HOST = 3;
  public static final int ERROR = 4;

  
  
  //  ChatClientApplet constructor
  //
  //  empty, as you can see.

  public ChatClientApplet() {
  }
  

  //  main
  //
  //  Construct the applet inside a frame, in the center of the screen

  public static void main(String[] args) {
    Security.addProvider(new com.sun.crypto.provider.SunJCE());
    System.setSecurityManager(new RMISecurityManager());
    
    ChatClientApplet applet = new ChatClientApplet();
    applet.isStandalone = true;
    Frame frame = new Frame();
    frame.setTitle("ChatRoom Applet");
    frame.setSize(464, 464);
    frame.add(applet, BorderLayout.CENTER);
    applet.init();
    applet.start();
    Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
    frame.setLocation((d.width - frame.getSize().width) / 2, 
		      (d.height - frame.getSize().height) / 2);
    frame.setVisible(true);
  }


  //  init
  //
  //  Initialize the applet.

  public void init() {
    try {
      jbInit();
    }
    catch (Exception e) {
      System.out.println("ChatClientApplet exception: " + e.getMessage());
      e.printStackTrace();
    }
  }
  

  //  jbInit
  //
  //  Component initialization

  private void jbInit() throws Exception {
    mLoginPanel = new ChatLoginPanel(this);
    mLoginPanel.setSize(new Dimension(416, 368));
    mSubscribePanel = new ChatSubscribePanel(this);
    mSubscribePanel.setSize(new Dimension(416, 368));
    mChatPanel = new ChatRoomPanel(this);
    mChatPanel.setSize(new Dimension(416, 368));

    this.setBackground(Color.lightGray);
    this.setSize(new Dimension(416, 368));
    
    this.setLayout(mLayout);
    this.add("ChatRoom", mChatPanel);
    this.add("Login", mLoginPanel);
    this.add("Subscribe", mSubscribePanel);
    mLayout.show(this, "Login");
  }
  

  //  destroy
  //
  //  Called when an applet is about to be garbage collected.  When we run
  // as an application, we call this directly when the "quit" button is
  // clicked. 

  public void destroy() {
    try {
      mServer.unregister(mClientID);
    } catch (Exception err) {
      System.out.println("ChatClientApplet exception: " +
			 err.getMessage());
      err.printStackTrace();
    }
    if (isStandalone) {
      System.exit(0);
    }
  }
    

  //  subscribe
  //
  //  Called from the subcribe panel when the user clicks the "subscribe"
  // button.

  public void subscribe(String host, int port, String name, 
			String handle, String password) {
    try {

    } catch (Exception e) {
	System.out.println("ChatClientApplet err: " + e.getMessage());
	e.printStackTrace();
    }
  }


  //  authenticate
  //
  //  Called from the login panel when the user clicks the "connect"
  // button.  If this user has not subscribed, show the subscribe panel
  // and return.  If they have subscibed, perform the authentication
  // protocol with the server.  Returning false will cause an
  // "Authentication failed" message to be displayed in the login panel.

  public int authenticate(String handle, String password, 
			  String host, int port) {
    try {

      //  We store the user's private authentication key in a file called
      // <handle>.<service>.priv.  This code detects if that already
      // exists and takes appropriate actions if it does not.
      //
      //      try {
      //	new FileInputStream(handle + ".Chat.priv");
      //      } catch (FileNotFoundException e) {
      //	mSubscribePanel.setParams(handle, password, host, port);
      //	mLayout.show(this, "Subscribe");
      //	return true;
      //      }
      
      mHandle = handle;
      mServer = (ChatServer)Naming.lookup("//" + host + ":" + 
					  port + "/ChatServer");
      mClientID = mServer.getClientID();
      if (!doAuthentication(handle, password)) {
	return AUTHENTICATION_FAILED;
      }
      
      mConsumer = new ChatConsumerImpl(mChatPanel.getOutputArea());
      mServer.register(mConsumer, mClientID);
      
      mLayout.show(this, "ChatRoom");
      return SUCCESS;
    } catch (AccessControlException accE) {
      return BAD_HOST;
    } catch (RemoteException remoteE) {
      return CONNECTION_REFUSED;
    } catch (Exception e) {
      System.out.println("ChatClientApplet err: " + e.getMessage());
      e.printStackTrace();
    }
    
    return ERROR;
  }
  

  //  doAuthentication
  //
  //  Actually do the protocol with the server, return whether it was
  // successful or not.

  public boolean doAuthentication(String handle, String password) {
    return true;
  }
  

  //  sendMessage
  //
  //  Called from the ChatPanel when the user types a carrige return.  You
  // will need to change this.

  public void sendMessage(String msg) {
    try {
      String str = mHandle + "> " + msg;
      mServer.postMessage(str, mClientID);
    } catch (Exception e) {
      System.out.println("ChatClientApplet err: " + e.getMessage());
      e.printStackTrace();
    }
  }


  //  getParameter
  //
  //  Get a parameter value.  Generated automatically by JBuilder

  public String getParameter(String key, String def) {
    return isStandalone ? System.getProperty(key, def) :
      (getParameter(key) != null ? getParameter(key) : def);
  }
  
  //  getAppletInfo
  //
  //  Get Applet information.  Generated automatically by JBuilder
  public String getAppletInfo() {
    return "Who gets Applet information?  You are so lame";
  }
  
  //  getParameterInfo
  //
  // Get parameter info.  Generated automatically by JBuilder
  public String[][] getParameterInfo() {
    return null;
  }
  

}

