//  ChatServerImpl.java
//
//  Last modified 2/17/99 by Alan Frindell
//
//  This class implements the ChatServer interface.
//
//  You need to add authentication and encryption code to this class.

package Authenticator.Chat;

import Authenticator.*;

import java.rmi.*;
import java.rmi.server.UnicastRemoteObject;

import java.io.*;
import java.util.*;
import java.math.BigInteger;

import java.security.*;
import java.security.spec.*;
import java.security.interfaces.*;
import javax.crypto.*;
import javax.crypto.spec.*;
import javax.crypto.interfaces.*;


public class ChatServerImpl extends UnicastRemoteObject
                            implements ChatServer {
  Hashtable mClients;
  Integer mNextID;
  Escrow mEscrow;
  
  //  ChatServerImpl constructor
  //
  //  Initializes some fields and connects to the escrow agent

  public ChatServerImpl(String hostName, int port) throws RemoteException {
    mClients = new Hashtable();
    mNextID = new Integer(0);
    
    try {
      //      mEscrow = (Escrow)Naming.lookup("//" + hostName + ":" + port + 
      //				      "/Escrow");
    } catch (Exception e) {
      System.out.println("ChatServerImpl err: " + e.getMessage());
      e.printStackTrace();
    }
  }
  
  
  //  main
  //
  //  Creates the one and only ChatServerImpl object and registers it with
  // RMI.

  public static void main(String args[]) {
    Security.addProvider(new com.sun.crypto.provider.SunJCE());
    System.setSecurityManager(new RMISecurityManager());
    
    int port;
    
    if (args.length != 2) {
      System.out.println("Useage: java ChatServerImpl host portNum");
      return;
    } else {
      try {
	port = Integer.parseInt(args[1]);
      } catch (NumberFormatException nfExp) {
	System.out.println("Useage: java ChatServerImpl host portNum");
	return;
      }
    }

    try {
      ChatServerImpl obj = new ChatServerImpl(args[0], port);
      Naming.rebind("//" + args[0] + ":" + port + "/ChatServer", obj);
      System.out.println("ChatServer bound in registry");
    } catch (Exception e) {
      System.out.println("ChatServerImpl err: " + e.getMessage());
      e.printStackTrace();
    }
  }
  
  
  //  getClientID
  //
  //  returns a unique ID to be used to distinguish between callers.  Note
  // that this is not really used in the insecure version, but is very
  // handy when adding security features.

  public synchronized Integer getClientID() throws RemoteException {
    Integer id = mNextID;
    mNextID = new Integer(id.intValue() + 1);
    return id;
  }


  //  keyExchange
  //
  //  Used by clients to establish a shared secret with the server.  This
  // is a big hint for the secure version.

  public PublicKey keyExchange(PublicKey pubKey, Integer clientID)
    throws RemoteException {
    try {

    } catch (Exception e) {
      System.out.println("ChatServerImpl: " + e.getMessage());
      e.printStackTrace();
    }
    return null;
  }


  //  register
  //
  //  Registers a callback object with the server.  When messages are
  // posted to the server, they are then sent to all of the registered
  // consumer objects.  These consumer objects are attached to the client
  // and display the message they receive in the Chat window.
    
  public void register(ChatConsumer consumer, Integer clientID) 
    throws RemoteException {

    mClients.put(clientID, consumer);
  }


  //  unregister
  //
  //  When a client leaves the chat room, remove their callback so we
  // don't keep sending messages to it after it dies.

  public void unregister(Integer clientID) 
    throws RemoteException {
    
    mClients.remove(clientID);
  }


  //  postMessage
  //
  //  Called by a client to post a message to the chat room.  Cycle
  // through all of the registered clients and send them the message we
  // got.  We leave a hook in here to determine if a message is
  // "malicious".  Note, that we don't really need the clientID for the
  // insecure version, but again, this will prove handy for adding
  // additional security features.

  public void postMessage(String msg, Integer clientID) 
    throws RemoteException {
    try { 
      ChatConsumer consumer;

      if (malicious(msg)) {
	//  name = mEscrow.getIdentity(...);
      }
	
      Enumeration theClients = mClients.elements();
      while (theClients.hasMoreElements()) {
	consumer = (ChatConsumer)theClients.nextElement();
	consumer.consumeMessage(msg);
      }
    } catch (Exception e) {
      System.out.println("ChatServerImpl err: " + e.getMessage());
      e.printStackTrace();
    }

  }


  //  malicious
  //
  //  returns true if the given string meets our criteria for "bad".  Feel
  // free to change this to meet your own criteria.

  private boolean malicious(String msg) {
    return (msg.indexOf("bomb") != -1);
  }


}
