//  EscrowImpl.java
//
//  Last Updated by Alan Frindell 2/16/99
//
//  The EscrowImpl implements the functionality of the escrow agent
// described in the protocol.  It generates keys for new services and new
// subscribers and has the capability to discover the name of a subscriber
// deemed "bad" by a service.


package Authenticator;

import java.rmi.*;
import java.rmi.server.UnicastRemoteObject;

import java.io.*;
import java.math.BigInteger;
import java.util.Hashtable;

import java.security.*;
import java.security.spec.*;
import java.security.interfaces.*;
import javax.crypto.*;
import javax.crypto.spec.*;
import javax.crypto.interfaces.*;


public class EscrowImpl extends UnicastRemoteObject
                        implements Escrow {

  String mPassword;
  KeyPair mDHKeyPair;


  //  EscrowImpl Constructor
  //  
  //  Generates a Diffie-Hellman key pair for the escrow

  public EscrowImpl(String password) throws RemoteException {
    try {
      mPassword = password;

      KeyPairGenerator dhKeyGen =
	KeyPairGenerator.getInstance("DiffieHellman"); 
      dhKeyGen.initialize(SkipConstants.sDHParameterSpec);
      mDHKeyPair = dhKeyGen.genKeyPair();

    } catch(Exception e) {
      System.out.println("EscrowImpl err: " + e.getMessage());
      e.printStackTrace();
    }
  }


  //  main
  //
  //  Creates the one and only EscrowImpl object and registers it with
  //  RMI.

  public static void main(String args[]) {
    Security.addProvider(new com.sun.crypto.provider.SunJCE());
    System.setSecurityManager(new RMISecurityManager());
    
    if (args.length != 3) {
      System.out.println("Useage: java escrow hostname port password\n");
      return;
    }

    try {
      EscrowImpl escrow = new EscrowImpl(args[1]);
      Naming.rebind("//" + args[0] + ":" + args[1] + "/Escrow", escrow);
      System.out.println("Escrow bound in registry");
      
    } catch (Exception e) {
      System.out.println("EscrowImpl err: " + e.getMessage());
      e.printStackTrace();
    }
  }
  
  
  //  initService
  //  
  //  Called by new services when they want the escrow to generate a
  // public/private key pair (N, T, s) / (R, p, t).  The public key should
  // be stored in a public file called <service>.pub.  The private key
  // should be encrypted using the escrow's password and stored in a file
  // called <service.priv>.

  public synchronized void initService(String service) 
    throws RemoteException { 
  }
  

  //  keyExchange
  //
  //  Called by subscribers who need to create a shared key with the
  // escrow.  Feel free to change this code if you like, we are merely
  // giving you an example of how the KeyAgreement class works.

  public synchronized PublicKey keyExchange(PublicKey x) 
    throws RemoteException {

    try {
      KeyAgreement exchange = KeyAgreement.getInstance("DiffieHellman");
      exchange.init(mDHKeyPair.getPrivate());
      exchange.doPhase(x, true);
      SecretKey desKey = exchange.generateSecret("DESede");
      return mDHKeyPair.getPublic();

    } catch (Exception e) {
      System.out.println("EscrowImpl err: " + e.getMessage());
      e.printStackTrace();
    }
    
    return null;
  }


  //  subscribe
  //
  //  Called by a subscriber to get an authentication key for the given
  // service.  Note that both the parameters and the return value are
  // encrypted with some shared key.  You may want to change the
  // parameters to this call to allow multiple people to subscribe
  // simultaneously (though you don't have to).
  //  This function also places i and username into the
  // <service>.subscribers file, which is encrypted with the escrow's
  // password.  i is the number of keys issued so far by the escrow for
  // the given service.

  public synchronized byte[] subscribe(byte[] encRealName, 
				       byte[] encServiceName) 
    throws RemoteException {
    return null;
  }
  

  //  getPublicKeyFor
  //
  //  Returns the (N, T, s) for a given service.

  public AuthenticatorPublicKey getPublicKeyFor(String service) 
    throws RemoteException {
    return null;
  }


  //  getSignatureKey
  //
  //  This simply returns a public key that can be used to verify the
  // escrow's signature on a file.

  public PublicKey getSignatureKey() throws RemoteException {
    return null;
  }



  //  getIdentity
  //
  //  Called by a service to recover the username of a user who is
  // misbehaving.  Z part of the record of the user's authentication to
  // the service.

  public synchronized String getIdentity(String service, BigInteger Z) 
    throws RemoteException {

    return "Alice";
  }
      
}
